;; Nodes are turtles -- and the edges connecting them are
;; turtles also!
breeds [nodes edges]

;; Edges are directional.  They go FROM node A, TO node B.
;; The "a" and "b" variables each store a node.
edges-own [a b]

;; Each node keeps track of edges going out, and edges coming
;; in.  The "out-edges" and "in-edges" variable each store
;; lists of edges.  (We can't easily add and remove things
;; from agentsets, so we use lists instead.)
nodes-own [out-edges in-edges]

to setup
  ca
  ;; These shapes were drawn using the Shapes Editor, on
  ;; the Tools menu.
  set-default-shape nodes "circle"
  set-default-shape edges "line"
  cct-nodes number-of-nodes
  [
    ;; Initially, nodes have no connecting edges.  So we set
    ;; out-edges and in-edges to the empty list.
    set out-edges []
    set in-edges []
    set color blue
    set size 3
    ;; Form a circle.
    fd screen-edge-x - 2
  ]
end

;; This procedure demos the creation and deletion of edges.
;; Here we create and delete edges totally at random, except that
;; we never go below 10 edges.
to go
  ask random-one-of nodes
    [ form-random-edge ]
  if count edges > number-of-edges
    [ ask random-one-of edges
        [ disappear ] ]
end

to form-random-edge ;; node procedure
  locals [other-node]
  ;; pick a random other node
  set other-node random-one-of nodes with [self != myself]
  ;; make sure we got another node, and that we aren't connected
  ;; to it, and that it isn't connected to us
  if (other-node != nobody) and
     (not connected-to? other-node) and
     (not value-from other-node [connected-to? myself])
    [ connect-to other-node ]
end

;; Helper procedure for checking whether this node is already
;; connected to a given other node.  Note the use of MAP to
;; get all the nodes that are at the other end of this node's
;; out-edges.
to-report connected-to? [other-node]  ;; node procedure
  report member? other-node (map [b-of ?] out-edges)
end

;; This makes a new edges that leads from node, to a given
;; other node.
to connect-to [other-node]  ;; node procedure
  ;; The HATCH command is how a turtle makes another turtle.
  ;; Initially, the child turtle is an exact duplicate of the
  ;; parent and inherits all of the parent's values for its
  ;; turtle variables.  Anything we want to be different about
  ;; the child, we then change.
  hatch 1
    [ ;; Change our breed and color.
      set breed edges
      set color green
      ;; Here "myself" refers back to our parent.  Remember
      ;; that the directionality of the edge is from A to B.
      set a myself
      set b other-node
      ;; Now update the edge lists of both nodes so it includes
      ;; this new edge.
      set (out-edges-of a) lput self (out-edges-of a)
      set (in-edges-of b) lput self (in-edges-of b)
      reposition ]
end

;; This procedure is used to make an edge be the correct size
;; and position.  We call it at the time we create an edge.  If
;; you modify this example so that nodes can move, then you will need
;; to call this procedure on all of the edges connected to a node
;; so they remain visually connected to the node.
to reposition  ;; edge procedure
  ;; To make the edge appear to connect the two nodes, we need to
  ;; put the edge at the midpoint between the two nodes.  The easiest
  ;; way to get to the midpoint is to start on top of node A...
  setxy (xcor-of a) (ycor-of a)
  ;; Then turn so we point towards node B.
  set heading towards-nowrap b
  ;; Make our size equal to the distance between the two nodes!
  set size distance-nowrap b
  ;; Then, to reach the midpoint, we move forward exactly half of
  ;; the distance to B.
  jump (distance-nowrap b) / 2
end

;; Calling this procedure makes an edge delete itself.
to disappear  ;; edge procedure
  ;; First remove ourselves from the edges lists of the two nodes
  ;; we connect.
  set (out-edges-of a) remove self (out-edges-of a)
  set (in-edges-of b) remove self (in-edges-of b)
  ;; Now we can safely expire.
  die
end


; (C) 2004 Uri Wilensky.  This code may be freely copied, distributed,
; altered, or otherwise used by anyone for any legal purpose.
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
; OWNERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
@#$#@#$#@
GRAPHICS-WINDOW
196
10
616
451
20
20
10.0
1
10
1
1
1

BUTTON
57
180
140
213
NIL
go
T
1
T
OBSERVER
NIL

SLIDER
10
40
189
73
number-of-nodes
number-of-nodes
0
100
30
1
1
NIL

BUTTON
62
82
139
115
NIL
setup
NIL
1
T
OBSERVER
T

CC-WINDOW
71
454
614
550
Command Center

SLIDER
9
218
188
251
number-of-edges
number-of-edges
0
100
20
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This example demonstrates how to make a network in NetLogo.  The network consists of a collection of nodes, some of which are connected by edges.

This example doesn't do anything in particular with the nodes and edges.  You can use it as the basis for your own model that actually does something with them.


EXTENDING THE MODEL
-------------------
Try making it so you can drag the nodes around using the mouse.

Position the nodes in a circle, instead of randomly.

Use the turtle variable LABEL to label the nodes and/or edges with some information.

Try calculating some statistics about the network that forms, for example the average degree.

Try other rules for connecting nodes besides totally randomly.  For example, you could:
- Connect every node to every other node.
- Make sure each node has at least one edge going in or out.
- Only connect nodes that are spatially close to each other.
- Make some nodes into "hubs" (with lots of edges).
And so on.

Make two kinds of nodes, differentiated by color, then only allow edges to connect two nodes that are different colors.  This makes the network "bipartite."  (You might position the two kinds of nodes in two straight lines.)


NETLOGO FEATURES
----------------
The nodes use lists to keep track of their edges.

In the GO button, the "Force display update after each iteration" checkbox is turned off.  This makes the model run a lot faster, since we don't need to see each edge appear and disappear individually.


RELATED MODELS
--------------
Owen Densmore's "Layout" model, available from the NetLogo User Community Models web page (http://ccl.northwestern.edu/netlogo/models/community/), shows how to make the nodes and edges automatically reposition themselves into a pleasing-looking layout.  It also shows how to generate a network with "hubs", where the distribution of the number of edges the nodes have follows a power law.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

line
true
0
Line -7566196 true 150 0 150 301

@#$#@#$#@
NetLogo 2.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
