breeds [
  reactants      ;; reactant molecules (NO2), shown as green
  products       ;; product molecules (N2O4), shown as red
]

turtles-own [
  reacting?      ;; used to ensure that reactants can only participate in one reaction at a time
]

globals [
  Kf                 ;; when temp-effect is on, replaces Kb
  Kr                 ;; when temp-effect is on, replaces Ku
  Keq                ;; equilibrium constant
  react-conc         ;; concentration of reactant
  prod-conc          ;; concentration of product
  black-patches      ;; agentset of all the patches inside the reaction container
]

to setup
  ca
  set-default-shape reactants "NO2"
  set-default-shape products "N2O4"
  draw-box                                        ;; draws the reaction container
  create-custom-reactants 200                     ;; generates reactants
    [ set color green
      set reacting? false
      randomize ]                                 ;; distributes reactants across the screen
  update-concentrations
  do-plotting
end

to update-concentrations
  set react-conc (count reactants / count black-patches) * 100
  set prod-conc (count products / count black-patches) * 100
end

to go
  ifelse temp-effect?
    ;; if temp-effect is ON, Kf  replaces Kb and Kr replaces Ku
    ;; Kf and Kr are calculated with the Arrhenius Equation, k = ln A - Ea / RT .
    ;; ln A, Ea, and R are all constants for a reaction. Below, the constants have simulated 
    ;; values that were chosen to produce the same qualitative results as the real-world
    ;; reaction. Because the number of molecules in the model is so small, we use simulated
    ;; constants to keep the values of each variable in a useful range for the model.
    ;; The important thing to note is how each K value varies according to the
    ;; temperature of the reaction.
    [ set Kf (5 + (3000 /(8 * (temp + 273))))
      set Kr (15 - (8000 / (8 * (temp + 273)))) ]
    ;; if temp-effect is OFF, set Keq based on concentrations
    [ set Keq (prod-conc / (react-conc ^ 2)) * 100 ]
  ask turtles
    [ bounce                        ;; bounce off walls
      fd 1                          ;; wander aimlessly
      rt random-float 10 - random-float 10 ]
  ask reactants
    [ react-forward ]
  ask products
    [ react-backward ]
  update-concentrations
  do-plotting
end

;; see explanations of "react-forward" and "react-backward" in the info window

to react-forward  ;; turtle procedure
  locals [partner chance]
  ifelse temp-effect?
    [ set chance Kf ]   ;; multiply Kb by 2 because 2 molecules are involved
    [ set chance Kb ]  ;; use of 1.0 here is arbitrary because calculations for real Kb would be too difficult
  if (any? other-reactants-here) and (random-float 10.0 < chance)
    [ set reacting? true  ;; prevent anyone else from trying to react with us
      set partner (random-one-of other-reactants-here)
      if not reacting?-of partner
        [ set reacting?-of partner true  ;; prevent anyone else from trying to react with our partner
          set breed-of partner products
          set reacting?-of partner false
          set color-of partner red
          die ]
      set reacting? false ]
end

to react-backward  ;; turtle procedure
  locals [chance]
  ifelse temp-effect?
    [ set chance Kr ]
    [ set chance Ku ]
  if (random-float 1000.0 < chance)
    [ set breed reactants
      set color green
      hatch 1
        [ rt 180 ] ]
end

;; turtle procedure to bounce molecules off the yellow walls
to bounce
  locals [box-edge]
  set box-edge (screen-edge-x - edge-size)  ;; setting this first makes the calculations a bit simpler
  ; check: hitting top or bottom wall?
  if ((pycor = box-edge)       and ((heading > 270) or (heading < 90))) or
     ((pycor = (- box-edge)) and ((heading > 90) and (heading < 270)))
    ; if so, reflect heading around y axis
    [ set heading (180 - heading) ]
  ; check: hitting left or right wall?
  if ((pxcor = box-edge)       and ((heading > 0) and (heading < 180))) or
     ((pxcor = (- box-edge)) and ((heading > 180)))
    ; if so, reflect heading around x axis
    [ set heading (- heading) ]
end

;;observer procedures to add more molecules to the model
to add-reactant
  create-custom-reactants 20
    [ set color green
      set reacting? false
      randomize ]
end

to add-product
  create-custom-products 20
    [ set reacting? false
      set color red
      randomize ]
end

;;;; plotting procedures

to do-plotting
  set-current-plot "Concentrations"
  set-current-plot-pen "Reactant"
  plot react-conc
  set-current-plot-pen "Product"
  plot prod-conc
end

to draw-box
  ask patches
    [ ifelse (abs pxcor > (screen-edge-x - edge-size)) or
             (abs pycor > (screen-edge-y - edge-size))
        [ set pcolor yellow ]
        [ set pcolor black ] ]
  set black-patches patches with [pcolor = black]
  ask turtles
    [ randomize ]
end

;; turtle procedure that places turtles randomly within the box
to randomize
  locals [random-patch]
  set random-patch random-one-of black-patches
  setxy (pxcor-of random-patch) (pycor-of random-patch)
  rt random-float 360
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo Simple Kinetics 2 model.
; http://ccl.northwestern.edu/netlogo/models/SimpleKinetics2.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/SimpleKinetics2
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
255
10
640
416
12
12
15.0
1
10
1
1
1

CC-WINDOW
255
419
630
519
Command Center

BUTTON
4
41
79
74
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
84
41
153
74
NIL
go
T
1
T
OBSERVER
T

BUTTON
5
248
101
281
redraw box
draw-box
NIL
1
T
OBSERVER
T

SLIDER
5
214
101
247
edge-size
edge-size
1
10
1
1
1
NIL

SWITCH
116
214
251
247
temp-effect?
temp-effect?
1
1
-1000

SLIDER
3
106
160
139
Kb
Kb
0.0
10.0
10.0
0.1
1
NIL

SLIDER
3
140
160
173
Ku
Ku
0.0
100.0
34.0
1.0
1
NIL

BUTTON
169
140
252
173
add reactant
add-reactant
NIL
1
T
OBSERVER
T

BUTTON
169
106
252
139
add product
add-product
NIL
1
T
OBSERVER
T

SLIDER
116
248
251
281
temp
temp
-200.0
200.0
200.0
1.0
1
deg C

MONITOR
13
289
122
338
Reactant Conc
react-conc
1
1

MONITOR
123
289
226
338
Product Conc
prod-conc
1
1

PLOT
4
339
247
519
Concentrations
time
# molecules
0.0
10.0
0.0
10.0
true
true
PENS
"Reactant" 1.0 0 -11352576 true
"Product" 1.0 0 -65536 true

TEXTBOX
5
184
102
213
Volume (pause model to change)

TEXTBOX
165
88
255
106
Concentration

TEXTBOX
117
196
207
214
Temperature

TEXTBOX
4
88
161
106
Rates (when temp-effect off)

@#$#@#$#@
WHAT IS IT?
-----------
This model demonstrates the properties of LeChatelier's Principle. This chemical principle states that if a system that is at equilibrium is perturbed, the system will readjust to establish a new equilibrium. For example, if you add reactants to a reversible reaction that is at equilibrium, the system will shift to generate more products and establish a new equilibrium. The principle can also be described with chemical equations.

Below is a generic equation which depicts two molecules of reactant A combining to form one molecule or product B. The reaction is reversible, meaning that the one molecule of B can break down into two molecules of A.

|                  Kb
|        A + A <=======> B
|                  Ku

An example of such a reaction would be dimerization of the gas nitrous oxide:

|                  Kb
|        2 NO  <=======> N O
|            2     Ku     2 4

This reaction is an example of a complex reaction which consists of two elementary reactions.  The forward bimolecular reaction

|                 Kb
|        A + A --------> B

is characterized by the constant Kb and the reverse unimolecular reaction

|              Ku
|        B ---------> A + A

The equilibrium rate constant for the entire reaction (Keq) is equal to [B] / [A] ^ 2. Each of the rate constants in the equations above has units of s^-1. They are empirically derived constants that when combined with the reaction concentrations tell you how fast the reaction proceeds according to the reaction rate law. The rate law ultimately tells you how many Molar units of a molecule react per second. For the reaction above the forward rate law is RATE = Kb[A]^2 and the reverse rate law is RATE = Ku[B].

Note that because we are simulating the reaction, the values of Kb and Ku in this reaction are not real-world values.  It would be necessary to use several differential equations to calculate the real values of Kb, Ku and Keq, however, several qualitative features of their relationships can be seen using this model. Reaction equilibrium is reached when a system reaches a steady-state. This is not to say that reactions have stopped occurring!  Microscopic changes in equilibrium still take place, but to our eyes and our measurements the system appears stable because the forward and reverse rates are equal.

The rate at which a reaction reaches equilibrium as well as the state of the equilibrium system both depend upon the rate constants, the temperature, the concentration of reactants and products and, when a gas is involved, the volume of the container. When a system has reached equilibrium, changes to any of the variables above result in a change in the system to establish a new equilibrium. This effect is predicted using LeChatelier's Principle. We can use our model to discover the role of each variable (temperature, volume, concentration and rate constant) in LeChatelier's Principle.


HOW TO USE IT
--------------
To start off:

Choose the values of Kb and Ku with appropriate sliders:
- Kb controls the rate of the forward reaction by which two green molecules turn bimolecularly into a single red molecule.
- Ku controls the rate of the reverse reaction, by which a red molecule turns unimolecularly into two green molecules.

Having chosen appropriate values of the constants, press SETUP to clear the screen and create an initial number of green molecules.  Note: we do not create red molecules initially, although this can be done in principal.

Press RUN to start the simulation.

Set the size of the yellow box using the EDGE-SIZE slider. (If you would like to change the size while you are running a model. Press RUN to stop the model, adjust the EDGE-SIZE slider and redraw the box using the REDRAW BOX button. Resume the reaction by pressing RUN.)

After viewing the effects of several different rate constant values, use the other sliders and buttons to observe how concentration, volume, and temperature affect the equilibrium.

A note on the temperature variable. Temperature changes have a unique effect on equilibrium compared with the other variables. You can observe this effect by toggling the TEMP-EFFECT button on or off and using the slider to set the temperature of the reaction in centigrade.


THINGS TO NOTICE
----------------
You will see molecules wandering around the screen and changing color.  Pay more attention to the plot of the concentrations.  Do the plots soon reach stationary concentrations?

How does changing the concentrations of reactants and products in the system affect the equilibrium? Does it take more or less time to reach a stationary condition under various conditions?

What is the effect of temperature on the equilibrium of the system compared to volume or concentration?  In the Procedures window, note how rate constants are calculated based on the temperature.

Notice how the ratio of products to reactants changes with changes to the system. Does the ratio change much with each factor? Make a window that show the value of Keq to help you determine this.

Why do the traces of each breed eventual balance around a constant average? How come this value is an average and not a constant?


THINGS TO TRY
-------------
How do the stationary concentrations depend on the values of Kb and Ku?   You can change Ku and Kb while the model is running.   See if you can predict what the stationary concentrations will be with various combinations of Kb and Ku.

Without adding additional reactants or products and with the temperature effect in the off position, note that more red product molecules accumulate when the volume decreases.  Can you explain why?

Observe the progress of the reaction at high and low temperatures.  Does this observed trend fit your expectations?

Try adding some molecules to the system that have no "breed", as an inert gas. Does this affect the equilibrium? Why or why not?


EXTENDING THE MODEL
------------------
Try altering the code so that when two green molecules collide, they produce two red molecules instead of one. Likewise, alter it so that two red molecules must collide to form two green molecules. Observe the effect of volume on this system. Is the effect as you predicted?

What would the effect of adding a catalyst to the system be?  Add a catalyst breed that accelerates the reaction and observe the trend.  Are you surprised?

Add a monitor that measures the equilibrium constant for the system.  Is it really a constant?


RELATED MODELS
--------------
Simple Kinetics 1, Simple Kinetics 3.  Note that the original StarLogoT version of this model was named "Le Chatelier 1".


NETLOGO FEATURES
----------------
Notice the use of breeds in the model.

Notice how we store an agentset of patches in the BLACK-PATCHES variable.  Computing this agentset once ahead of time (at the time the box is drawn) is faster than recomputing it at every iteration.

It is a little tricky to ensure that a reactant never participates in two reactions simultaneously.  The turtle variable REACTING? is used to control this.  In the future, we will add a new primitive to NetLogo to make this easier.


CREDITS AND REFERENCES
----------------------
Thanks to Mike Stieff for his work on this model.

To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo Simple Kinetics 2 model. http://ccl.northwestern.edu/netlogo/models/SimpleKinetics2. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2001 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/SimpleKinetics2 for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 300 300 150 195 150 195 7 105 7 105 150 105 150 0 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

circle
true
0
Circle -7566196 true true 73 75 153

n2o4
true
0
Circle -7566196 true true 61 107 88
Circle -7566196 true true 151 106 88
Circle -1 true false 20 64 64
Circle -1 true false 217 63 64
Circle -1 true false 216 171 63
Circle -1 true false 21 173 61

no2
true
0
Circle -7566196 true true 104 104 91
Circle -1 true false 194 119 63
Circle -1 true false 46 120 58

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 300 120 165 45 45 75 60 150 120 225 60 255 45 180 165

thin-arrow
true
0
Polygon -7566196 true true 150 300 300 150 180 150 180 4 120 4 120 150 120 150 0 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
